@@ -47,8 +47,8 @@ class Config(JSONConf):
             'api_rel_path': 'api/v2',
             'async': True,
             'auto_login': True,
-            'batch_size': 1000,
-            'cubes_path': '~/.metrique/cubes',
+            'batch_size': 5000,
+            'sql_batch_size': 1000,
             'debug': None,
             'host': '127.0.0.1',
             'journal': False,
@@ -58,9 +58,12 @@ class Config(JSONConf):
             'password': None,
             'port': 5420,
             'retries': 1,
+            'sort': -1,
             'ssl': False,
             'ssl_verify': True,
             'username': os.getenv('USER'),
+            'cube_pkgs': ['cubes'],
+            'cube_paths': [],
         }
         super(Config, self).__init__(config_file=config_file, *args, **kwargs)
 
