@@ -20,7 +20,7 @@ and more.
     >>> g.ping()
     pong
     >>> ids = g.extract(uri='https://github.com/drpoovilleorg/metrique.git')
-    >>> q = c.query.fetch('git_commit', 'author, committer_ts')
+    >>> q = c.query.find('git_commit', 'author, committer_ts')
     >>> q.groupby(['author']).size().plot(kind='barh')
     >>> <matplotlib.axes.AxesSubplot at 0x6f77ad0>
 
@@ -99,7 +99,6 @@ class HTTPClient(object):
     query_history = history = query_api.history
     query_deptree = deptree = query_api.deptree
     query_count = count = query_api.count
-    query_fetch = fetch = query_api.fetch
     query_distinct = distinct = query_api.distinct
     query_sample = sample = query_api.sample
     query_aggregate = aggregate = query_api.aggregate
@@ -116,34 +115,30 @@ class HTTPClient(object):
                 <type HTTPClient(...)>
         '''
         if 'cube' in kwargs and kwargs['cube']:
-            try:
-                cube_cls = get_cube(kwargs['cube'])
-            except ImportError:
-                cube_cls = cls
+            cls = get_cube(cube=kwargs['cube'], init=False)
         else:
-            cube_cls = cls
-        return object.__new__(cube_cls)
+            cls = cls
+        return object.__new__(cls)
 
-    def __init__(self, config_file=None, cube=None, owner=None, **kwargs):
+    def __init__(self, cube=None, config_file=None, owner=None,
+                 name=None, **kwargs):
         self._config_file = config_file
         # all defaults are loaded, unless specified in
         # metrique_config.json
         self.load_config()
 
-        if isinstance(cube, basestring):
-            self.set_cube(cube)
-        elif cube:
-            raise TypeError(
-                "expected cube as a string, got %s" % type(cube))
-
-        self.owner = owner or self.config.username
-
         # update config object with any additional kwargs
         # passed in by the user
         for k, v in kwargs.items():
             if v is not None:
                 self.config[k] = v
 
+        self.owner = owner or self.config.username
+
+        if name:
+            # override the cube name
+            self.name = name
+
         # keep logging local to the cube so multiple
         # cubes can independently log without interferring
         # with each others logging.
@@ -153,6 +148,131 @@ class HTTPClient(object):
         # load a new requests session; for the cookies.
         self._load_session()
         self._auto_login_attempted = False
+        self._cache = {}
+
+    def __getitem__(self, name):
+        if isinstance(name, slice):
+            op = 'in'
+            one = False
+        else:
+            op = '=='
+            one = True
+
+        result = self.find('_oid %s %s' % (op, json.dumps(name)),
+                           fields='__all__',
+                           raw=True, one=one)
+        if not result:
+            if op == 'in':
+                result = []
+            else:
+                result = {}
+        return result
+
+    def __setitem__(self, name, value):
+        raise NotImplementedError
+
+    def __delitem__(self, name):
+        raise NotImplementedError
+
+    def __len__(self):
+        return self.count()
+
+    def __iter__(self):
+        return self
+
+    def insert(self, objs):
+        self.save(objs)
+
+    def extend(self, objs):
+        self.save(objs)
+
+    def next(self):
+        count = self._cache.get('counter', None)
+        if count is None:
+            k = self._cache['count'] = self.count()
+            count = self._cache['counter'] = 0
+            self._cache['sort'] = self.config.sort
+        else:
+            k = self._cache['count']
+        if count < k:
+            docs = self.find(sort=[('_oid', self._cache['sort'])],
+                             skip=count,
+                             limit=self.config.batch_size,
+                             raw=True,
+                             fields='__all__')
+            self._cache['counter'] += len(docs)
+            # FIXME: return back a generator rather than tuple?
+            return docs
+        else:
+            del self._cache['counter']
+            del self._cache['count']
+            del self._cache['sort']
+            raise StopIteration
+
+    def __getslice__(self, i, j):
+        return self.find(sort=[('_oid', self.config.sort)],
+                         raw=True, fields='__all__',
+                         skip=i, limit=j)
+
+    def __contains__(self, item):
+        return bool(self.count('_oid == %s' % item))
+
+    def keys(self):
+        i = 0
+        size = self.config.batch_size
+        for j in xrange(size, self.count(), size):
+            objs = self[i:j]
+            for o in objs:
+                yield o['_oid']
+                i = j
+        else:
+            objs = self[i:self.count()]
+            for o in objs:
+                yield o['_oid']
+
+    def values(self):
+        i = 0
+        size = self.config.batch_size
+        for j in xrange(size, self.count(), size):
+            objs = self[i:j]
+            for o in objs:
+                yield o
+                i = j
+        else:
+            objs = self[i:self.count()]
+            for o in objs:
+                yield o
+
+    def items(self):
+        i = 0
+        size = self.config.batch_size
+        for j in xrange(size, self.count(), size):
+            objs = self[i:j]
+            for o in objs:
+                yield o['_oid'], o
+                i = j
+        else:
+            objs = self[i:self.count()]
+            for o in objs:
+                yield o['_oid'], o
+
+    #def update(self, name)
+    # ...
+
+    # rather ... for listing attributes...
+    #def keys(self, **kwargs):
+    #    objs = self.query_sample(fields='__all__',
+    #                             raw=True, **kwargs)
+    #    keys = []
+    #    for o in objs:
+    #        keys.extend(o.keys())
+    #        keys = set(keys)
+    #    return sorted(keys)
+
+    #def __getattribute__(self, name):
+    #    # get fields on .attr
+    #    objs = self.find(fields='__all__',
+    #                     raw=True, **kwargs)
 
     def activity_get(self, ids=None):
         '''
@@ -292,8 +412,7 @@ class HTTPClient(object):
         ' wrapper for utils.get_cube(); try to load a cube, pyclient '
         config = copy(self.config)
         config.update(kwargs)
-        return get_cube(cube=cube, init=init, config=config,
-                        path=self.config.cubes_path)
+        return get_cube(cube=cube, init=init, config=config)
 
     def get_last_field(self, field):
         '''
@@ -420,14 +539,6 @@ class HTTPClient(object):
             else:
                 return json.loads(_response.content)
 
-    def set_cube(self, cube):
-        '''
-        give this instance a "cube name"; the cube name is expected
-        to exist already in the metrique host being interacted with,
-        or the cube needs to be registered.
-        '''
-        self.name = cube
-
     def whoami(self, auth=False):
         ' quick way of checking the username the instance is working as '
         if auth:
